# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package DisplayTable;

use strict;
use ParamParser;
use File::Basename;
use AnotherTemplate;
use O2DBI::O2DBI_WebUtils;
use O2DBI::DisplayField;

use LipmObject;
use base qw(LipmObject);

use Error qw( :try );
use LipmError;

=head2 Function _Init (the constructor)
 Title            : _Init
 Usage            : my $o_obj = New DisplayTable($o_cfg, $o_logger)
 Function         : threw the New function of LipmObject, builds the object
 Returns          : The object
 Args             : $o_cfg   = A ParamParser containing the configuration parameters
                    $o_logger= A Logger object (optional)
 Globals          : none

=cut
sub _Init
{
    my $self = shift;
    my ($o_cfg, $o_logger,$o_param) = (@_);
    
    $self -> {_o_cfg}   = $o_cfg;
    $self -> SetLogger($o_logger) if (defined $o_logger);
    $self -> {_o_param}   = $o_param;
    
    return $self;
}

=head2 Procedure SetTableName, Function GetTableName
 Title  : SetTableName, GetTableName
 Usage  : Setter, Getter

=cut

sub SetTableName
{
    my $self = shift;
    
    my ($table_name) = (@_);
    $self->{__table} = $table_name;
}
sub GetTableName
{
    my $self = shift;
    
    die "the table should be defined" unless exists $self->{__table};
    return $self->{__table};
}
sub GetDisplayField
{
    my $self = shift;
    
    die "the display_field object should be defined" unless exists $self->{__o_display_field};
    return $self->{__o_display_field};
}

=head2 Function GetTable
 Title        : GetTable
 Usage        : my $html_text = $o_obj->GetTable($html_template_file,$ra_rh_tuples);
 Prerequisite : none
 Function     : using a template, display O2DBI fetch results
 Returns      : $html_text : HTML to be displayed
 Args         : $template  : a template for use with AnotherTemplate
                $ra_rh_tuples : ARRAY REF of O2DBI fetch results
=cut

sub GetTable
{
    my $self = shift;
    
    my ($template, $ra_rh_tuples) = (@_);
    my $output_html="";
    my $count = @$ra_rh_tuples+0;
    if ($count==0)
    {
        return "<div class=\"centered\"><p><i>no match</i></p></div><hr />";
    };

    my $o_cfg = $self->{_o_cfg};
    my $img_http_dir = $o_cfg->Get('SERVERNAME') . $o_cfg->Get('HTTP_ROOT') . $o_cfg->Get('HTTP_IMG_DIR');
    
    # La barre de navigation
    my $o_navbar_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('navbar');
    $o_navbar_tmpl->AddVariable('IMG_HTTP_DIR',$img_http_dir);

    # Le template principal    
    my $o_main_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate($template);
    my $o_dspl      = $self->{__o_display_field};
    $o_main_tmpl->AddObject('DISPLAYFIELD',$o_dspl);
    $o_main_tmpl->AddObject('CFG',$o_cfg);

    my $i = 1;
    foreach my $rh_row (@{$ra_rh_tuples})
    {
        $o_dspl->SetRow($rh_row);

        $o_navbar_tmpl->AddVariable('ID_PREVIOUS',($i==1)?'wb_content':'ID_C'.($i-1));
        $o_navbar_tmpl->AddVariable('ID_NEXT',    ($i==$count)?'ID_SEARCH':'ID_C'.($i+1));
        $o_navbar_tmpl->AddVariable('COUNTER',$i);
        $o_navbar_tmpl->Parse();

        $output_html .= $o_navbar_tmpl->Get('parsed_template').$self->DisplayRow($o_main_tmpl,$rh_row);
        $i++;
    }
    
    return $output_html;
}        

=head2 Function DisplayRow
 Title        : DisplayRow
 Usage        : my $html_text = $o_obj->DisplayRow($template,$rh_row);
 OR             my $html_text = $o_obj->DisplayRow($o_template,$rh_row);
 Function     : using a template, display only 1 row of O2DBI fetch results
                If $template is a not a ref, it is considered as a template name and a new AnotherTemplate object is created
                If $template is a ref, it must be an already built AnotherTemplate object
 Returns      : $html_text : HTML to be displayed
 Args         : $template  : a template for use with AnotherTemplate
                $o_template: an anotherTemplate object
                $ra_rh_tuples : ARRAY REF of O2DBI fetch results
=cut

sub DisplayRow
{
    my $self = shift;
    
    my ($template, $rh_row) = (@_);
    my $o_main_tmpl;
    
    if (ref $template eq '')
    {
        my $o_dspl = $self->GetDisplayField();
        $o_dspl->SetRow($rh_row);
        $o_main_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate($template);
        $o_main_tmpl->AddObject('DISPLAYFIELD',$o_dspl);
    }
    else
    {
        $o_main_tmpl = $template;
    }
    
    $o_main_tmpl->AddVariable('ID',$rh_row->{id});

    return $o_main_tmpl->Parse();
}

=head2 Procedure PrintFormTable
 Title        : PrintFormTable
 Usage        : $o_obj->_PrintFormTable($o_wb);
 Procedure    : Print the results of the previous search, if any
                Unless in ajax mode, print the search form.
 Args         : $o_wb : A Webuilder object
 Access       : PROTECTED - Use this method from the derived classes only.
=cut

sub PrintFormTable
{
    my $self = shift;
    
    my ($o_wb) = (@_);
    my $o_param = $o_wb->GetObjectParam();
    my $input_template =  $self->GetTableName().'_search_form';
    my $output_template = $self->GetTableName().'_template';

    # Look for the previous search results, if any
    my $ra_o_main_table = $self->SearchData($o_param);
    #use Data::Dumper;
    #warn "PrintFormTable ".Dumper (%$ra_o_main_table->[0]);

    my $html_results    = $self->GetTable($output_template,$ra_o_main_table);
    
    # If ajax mode, just print the results
    if ($o_param->IsDefined('__wba_ajax') and $o_param->Get('__wba_ajax') ne '0')
    {
        print $html_results;
    }
    else
    {
        print '<div id="ID_RESULTS">'.$html_results.'</div>';
        
        # Print the form 
        my $html_form = &O2DBI::O2DBI_WebUtils::GetFormContent($input_template);
        print "<div class=\"HideOnFullScreen\" id=\"ID_SEARCH\">" . $o_wb -> GetForm("",$html_form) . "</div>";
    }
}


=head2 Function _SearchData
 Title        : _SearchData
 Usage        : my $ra_o_main_table = $o_obj->_SearchData($where_clause);
 or             my $ra_o_main_table = $o_obj->_SearchData($o_param);
 Function     : Search the data in the main table using the $where_clause, and looking for data in the joined tables
                If $o_param specified as parameter, the $where_clause is first computed
 Access       : PROTECTED METHOD - SHOULD BE WRAPPED BY SearchData IN THE DERIVED CLASSES
 Returns      : An array of tuples
 Args         : $where_clause (optional) = the clause to use to interrogate the database
 OR             $o_param                 = Parameters, they will be used to build the where clause
 TODO - REMPLACE souche_add::SearchData, MAIS TOUT N'EST PAS REPRIS !!!
=cut
sub _SearchData
{
    my $self = shift;
    
    my ($where_clause) = (@_);
    my $main_table = $self -> GetTableName();
    my $rh_joined_tables  = $self -> { _h_joined_tables };
    my $rh_joining_tables = $self -> { _h_joining_tables };

    # If a ParamParser were provided, compute the where clause
    if ((defined $where_clause) and (ref $where_clause eq 'ParamParser'))
    {
        my $o_param = $where_clause;
        $where_clause = &O2DBI::O2DBI_WebUtils::BuildWhereCondition($o_param,$main_table,$rh_joined_tables,$rh_joining_tables);
    }

    #if ($o_param->IsDefined('input_code'))
    #{
    #    $where_clause =
    #      "code ILIKE ('" . $o_param->Get('input_code') . "')";
    #}
    
    my $ra_o_main_table;
    if (defined $where_clause and $where_clause !~ /^ *$/)
    {
        $ra_o_main_table = &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL($where_clause, $main_table);
        my $nb_resultat = @$ra_o_main_table;
        $self->_Trace("displayTable::_SearchData calling O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL - where_clause = $where_clause, main_table = $main_table - resultat = $nb_resultat\n",'*','3');
    }
    else
    {
        @$ra_o_main_table=();
    }

    my ($ra_entete) = ();
    foreach my $joined_table (keys(%$rh_joined_tables))
    {
        my $joining_table    = $rh_joined_tables->{$joined_table}->{'joining_table'};
        ($ra_entete, $ra_o_main_table) =
          &O2DBI::O2DBI_WebUtils::N2Njoin($main_table, $joined_table, $joining_table, $ra_o_main_table);
    }
    
    # ???
    ($ra_entete, $ra_o_main_table) = &O2DBI::O2DBI_WebUtils::N21join($ra_o_main_table, $ra_entete);

    # Aller chercher les joining tables
    foreach my $joining_table (keys %$rh_joining_tables)
    {
        ($ra_entete, $ra_o_main_table) = &O2DBI::O2DBI_WebUtils::One2Njoin($main_table, $joining_table, $ra_o_main_table);
    }

    # ???
	
    ($ra_entete, $ra_o_main_table) = &O2DBI::O2DBI_WebUtils::N21join($ra_o_main_table, $ra_entete);
    return $ra_o_main_table;
}

1;


