# $Id$

=pod

=head1 clism::hash_table

clism::hash_table - manipulate a table like a hash

=head1 SYNOPSIS

   use clism::hash_table;

   my %some_hash;
   tie (%some_hash,"clism::hash_table","clism::some_object","clism::some_field")

=head1 DESCRIPTION

   If you tie your hash as written above, your tie is reflected to a database table, using o2dbi
   You must provide the constructor with the o2dbi object name (ie the table used to store the hash), 
   AND with the o2dbi field name (ie the column used to store the keys).
   THE VALUES RETRIEVED ARE THE COLUMN id
   
   When the hash is tied, the database is read, so that the hash is initialized with the table content
   When you try to fetch some key from the hash, two things may happen:
       1/ The key already exists in the database and in the hash: it is retrieved
       2/ The key does not exist: it is created, a new id is generated by o2dbi, and the key is stored in the hash and in the database
       
   IT IS ILLEGAL TO EXPLICITLY STORE SOMETHING TO THIS HASH
   
=cut

package hash_table;
use strict;
use Carp;

1;

=pod
=head2 TIEHASH

 Title          : TIEHASH
 Usage          : The constructor (see perltie for the details)
 Returns        : The object
 Errors         : dies if not enough paramters specified, or if o2dbi returns an error.
 Args           : the class, the o2dbi class to use, the field to use in the class.
 Globals        : none

=cut

sub TIEHASH 
{
    my $class      = shift;
    my $o2dbi_class= shift;
    my $o2dbi_field= shift;
    croak "ERROR - Not enough parameters" unless (defined($o2dbi_class) and defined($o2dbi_field));
    my $fetchall   = $o2dbi_class.'->fetchall()';
    my $ra_objects = eval($fetchall);
    if ($@ ne '')
    {
        croak "ERROR - $@";
    };
    my $self       = {};
    $self->{'_o2dbi_class'} = $o2dbi_class;
    foreach my $o_object (@$ra_objects)             # Read the data from the database, if any 
    {
        my $key = eval('$o_object->'.$o2dbi_field); 
        if ($@ ne '')
        {
            croak "ERROR - $@";
        };
        $self->{$key} = $o_object->id;
    };
    return bless ($self,$class);
};

=pod
=head2 FETCH, EXISTS, FIRSTKEY, NEXTKEY

 Usage          : see perltie
 Returns        : see perltie
 Errors         : dies if o2dbi returns an error.
 Args           : see perltie
 NOTE           : The string _o2dbi_class is USED by this object, 
                  SO THIS KEY CANNOT be a key of your tied hash.
                  This is a limitation, but is it a real problem ?
 Globals        : none

=cut

sub FETCH
{
    my $self = shift;
    my $key  = shift;
    if (!exists ($self->{$key}))
    {
        my $o2dbi_class = $self->{'_o2dbi_class'};
        my $o_object = eval("create $o2dbi_class(\$key)");
        if ($@ ne '')
        {
            croak "ERROR - $@";
        };
        &ObjError($o_object);
        $self->{$key} = $o_object->id;
    };
    return $self->{$key};
};

sub EXISTS
{
    my $self = shift;
    my $key = shift;
    return exists $self->{$key};
}

sub FIRSTKEY
{
    my $self = shift;
    my @a_keys = keys(%$self);       # reset each() iterator
    
    if ($a_keys[0] eq '_o2dbi_class')
    {
        each %$self;
    };
    return each %$self;
};

sub NEXTKEY
{
    my $self = shift;
    my $key = each %$self;
    if ($key eq '_o2dbi_class')
    {
        $key = each %$self;
    };
    return $key;
};

sub DELETE
{
    my $self = shift;
    my $key  = shift;
    if (exists ($self->{$key}))
    {
        my $o2dbi_class = $self->{'_o2dbi_class'};
        my $o_object = eval("create $o2dbi_class(\$key)");
        if ($@ ne '')
        {
            croak "ERROR - $@";
        };
        &ObjError($o_object);
        $o_object->delete();
    };
    return delete $self->{$key};
}
    
=pod
=head2 STORE, CLEAR

 Usage          : NOT IMPLEMENTED
                  Data are automagically stored when you fetch a new data
                  You should not delete anything from this hash/table
=cut

sub STORE 
{
    carp "ERROR - You should NOT store anything in this hash - It is automagically filled";
}
sub CLEAR 
{
    carp "ERROR - You should NEVER clear this hash";
}

=pod
=head2 ObjError

procedure: croak (NEVER RETURN) if the parameter is -1
           You pass ObjError an object created by o2dbi, it croaks if the object is not correctly initialized
=cut

sub ObjError
{
    croak "ERROR in creating an object" if (shift==-1);
};


