/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/

/* 

=pod

=head1 NAME

 LipmHistory.js - Objects useful for managing history of ajax requests
               LipmHistory objects are VERY useful in conjunction with the LipmAjax objects

=head1 SYNOPSIS

 LipmHistoryManagerBaseClass  An abstract class to implement a history manager
     LipmHistoryUlClass  A concrete class, using a <ul> to display the history
	 
 GLOBAL USED: O_PARAM 
              The following keys are used for LipmAjax/WebBuilder interaction:
              wba_ajax, wba_index, wba_start, wba_end

=head1 Classes and methods

=cut
*/

/*
=pod

=head1 class LipmHistoryManagerBaseClass

 Title        : LipmHistoryManagerBaseClass
 Usage        : Abstract class implementing the mechanism of a very simple history manager
                real history managers must derive from this class and add graphics capabilities.
 Constructor arg : initial_stamp A stamp (may be some parameter) which will be kept at start of the history
 Public member functions:
                Add  add an entry to the history
                IsAtStart return true if current position at start
                IsAtEnd   return true if current position at end
                GetPosition: return the current position of the history cursor
                Getstamp:    return the stamp of the current position
 Globals      : none

=cut
*/

LipmHistoryManagerBaseClass = Class.create({
	initialize: function(stamp)
	{
		// Create an array of bookmarks 		
		this.__history  = $A();
		
		// Keep the stamp as the first bookmark
		this.__history.push(stamp);

		// Keep The current position
		this.__current= 0;
	},
	
	/*
=pod

=head2 Add

	Title: Add
	Usage: From a derived class $super('stamp');
	Procedure: Remove all entries from the current position, then add a new entry 
	Args:      stamp The stamp to store in the new entry

=cut	
	*/
	Add: function(stamp)
	{
		// Remove all bookmarks from the last to the current (but excluding the current)
		for ( var i= this.__history.length-1; i> this.__current; --i) {this.__history.pop();};

		// Add the new entry		
		this.__history.push(stamp);
		
		return;
	},
	
	/*
=pod

=head2 IsAtStart, IsAtend

	Title: IsAtStart, IsAtend
	Usage: if (obj.IsAtStart())...
	Function: return true if the current position of the history is at the start/end 
	Args:  none
	Returns: true/false

=cut	
	*/
	IsAtStart: function() { return (this.__current == 0) ? true : false; },
	IsAtEnd:   function() { return (this.__current == this.__history.length-1) ? true : false; },
	
	/*
=pod

=head2 GetPosition

	Title: GetPosition
	Usage: var pos = obj.GetPosition()
	Function: return the current position 
	Args:  none
	Returns: the current position

=cut
	*/
	GetPosition: function() { return this.__current; },

	/*
=pod

=head2 GetStamp

	Title: GetStamp
	Usage: var pos = obj.GetStamp()
	Function: return the current stamp
	Args:  none
	Returns: the current stamp

=cut
	*/
	GetStamp: function() { return this.__history[this.__current]; },

	/*
=pod

=head2 _Mark

	Title: _Mark
	Usage: From a derived class $super('stamp');
	Function: Mark the 'index' entry as current 
	Args:     index The entry to mark
	Return:   false if the current was already at index, true if current position has changed
	Access:   protected
	
=cut
	*/
	_Mark: function(index)
	{
		if (index==this.__current) 
		{
			return false;
		}
		else
		{
			this.__current = index;
			return true;
		}
	}
});


/*
=pod

=head1 class LipmHistoryUlClass

 Title        : LipmHistoryUlClass
 Usage        : Concrete class, deriving from LipmHistoryManagerBaseClass, to manage the history as well as its visual effects
                The html code should be:    <div id="ID_HISTORY"><img src="history_prev.png" /><ul><li>&nbsp;</li></ul><img src="history_next.png" /></div>
                or simpler (but no arrows): <ul id="ID_HISTORY"><li>&nbsp;</li></ul>
 Constructor arg : top_elt The top element (div or ul as shown above)
                   initial_entry_name The inital entry name
                   initial_stamp      The initial stamp
 Globals      :  O_PARAM

=cut
*/

LipmHistoryUlClass = Class.create(LipmHistoryManagerBaseClass,{
	initialize: function($super,top_elt,initial_entry_name,initial_stamp)
	{
		// Initialize the superclass
		$super(initial_stamp);
		
		// Initialize private fields
		this.__top_elt = $(top_elt);
		if (this.__top_elt==null) {throw('ERROR - LipmHistoryUlClass - Couldn\'t find top_elt ' + top_elt);};
	
		var back_img   = this.__top_elt.down('img');
		var frwd_img   = this.__top_elt.down('img',1);

		// Keeping last, first and current element
		this.__last_li  = this.__top_elt.down('li');
		if (this.__last_li == undefined)
		{
			throw("ERROR - No <ul> or <ul> is empty");
		}
		this.__first_li = this.__last_li;
		this.__curr_li  = this.__first_li;
	    
		// Initialize first link
		var thisHistoryClass = this;
		this.__curr_li.innerHTML = '<a href="#">'+initial_entry_name+'</a>';
		this.__curr_li.addClassName('HistoryCurrent');
		this.__curr_li.addClassName('HistoryEntry');
		
		// Observe events
		Event.observe(this.__last_li,'click',function(evt){thisHistoryClass.Goto(evt.element().up('li'),0)});
		if (back_img != undefined)
		{
			Event.observe(back_img,'click',this.Back.bindAsEventListener(this));
			back_img.style.cursor='pointer';
			Event.observe(frwd_img,'click',this.Forward.bindAsEventListener(this));
			frwd_img.style.cursor='pointer';
		}
		
		// display the top element (should be hidden)
		this.__top_elt.show();
	},
	
	/* We need a ref to some ajax object, for the Goto/Reload methods */
	RegisterAjax: function(o_ajax)
	{
		this__o_ajax = o_ajax;
	},
	
	/*
=pod

=head2 Add

    Title: Add
    Usage: obj.Add('entry','stamp')
    Procedure: Remove all entries from the current position, then add a new entry 
    Args:  entry_name The name of this entry, will be displayed
           stamp      The stamp of this entry (managed by the superclass)
                      If undefined, the call is ignored

=cut
	*/
	Add: function($super,entry_name,stamp)
	{
		if (entry_name == undefined) {return;};

		// Call the superclass Add function
		$super(stamp);
		
		// Remove all bookmarks from the last to the current (but don't remove the current)
		var elt = this.__last_li;
		while ( elt != this.__curr_li )
		{
			var to_remove = elt;
			elt           = elt.previous();
			to_remove.remove();
		}
		this.__last_li = this.__curr_li;
		
		// Add a new bookmark
		var html='<li class="HistoryEntry"><a href="#">'+entry_name+'</a></li>';
		new Insertion.After(this.__last_li,html);

		// Update __last_li
		this.__last_li = this.__last_li.next();
		
		// Mark the new bookmark as current
		this._Mark(this.__last_li,this.GetPosition()+1)
		
		// Observe it
		var thisHistoryClass = this;
		var index = this.GetPosition();
		Event.observe(this.__last_li,'click',function(evt){thisHistoryClass.Goto(evt.element().up('li'),index)});
	},

	/*
=pod

=head2 Goto

    Title: Goto
    Usage: obj.Goto(elt,index)
    Procedure: Mark the element, index as "current" and go to the stored bookmark
    Args : elt  The element we want to mark, often produced through an event
           index The index of the position to mark
           NOTE - The elt is passed here for better performance, but it could be computed
    TODO: revoir cette histoire je pige plus rien
	
=cut
    */
	Goto: function(elt,index)
	{
		// this._Mark returns false if we were already at the current position, in this case there is nothing to do
		if (this._Mark(elt,index)==true)
		{
			// go to the wanted bookmark
			// this__o_ajax.SubmitAjax('read_bookmark='+this.GetStamp(),undefined,this.__GotoSuccess.bind(this));
			this__o_ajax.SubmitAjax(this.GetStamp(),undefined,this.__GotoSuccess.bind(this));
		}
	},

	/*
=pod

=head2 Reload

    Title: Reload
    Usage: obj.Reload()
    Procedure: Reload the history (current position)
    Args : none
	
=cut
    */
	Reload: function()
	{
		// go to the current bookmark
		this__o_ajax.SubmitAjax('read_bookmark='+this.GetStamp(),undefined,this.__GotoSuccess.bind(this));
	},
	
	__GotoSuccess: function()
	{
		InitForm(this__o_ajax.GetForm(),O_PARAM);
	},
	
	/*
	Title: _Mark
	Usage: obj._Mark(elt,index)
	Procedure: Mark the index as current calling the superclass function _Mark
	           Mark the elt as current, too
	Args:      elt The element we want to mark, often produced through an event
	           index The index of the position to mark
			   NOTE - The elt is produced for better performance, but it could be computed

    */
	_Mark: function($super,elt,index)
	{
		// Call the superclass _Mark function and return if nothing to do
		if ( $super(index) == false ) {return false;};
		
		// Change the current element
		this.__curr_li.removeClassName('HistoryCurrent');
		this.__curr_li = elt;
		this.__curr_li.addClassName('HistoryCurrent');

		return true;
	},
	
	/*
=pod

=head2 Back, Forward

    Title: Back, Forward
    Usage: obj.Back(), obj.Forward();
    Procedure: decrease or increase the current position then call Goto
               If already at start/end, do nothing
    Args: none
	
=cut
	
	*/
	
	Back: function()
	{
		if ( !this.IsAtStart() )
		{
			this.Goto(this.__curr_li.previous(),this.GetPosition()-1);
		}
	},
	Forward: function()
	{
		if ( !this.IsAtEnd() )
		{
			this.Goto(this.__curr_li.next(),this.GetPosition()+1);
		}
	}
});


