/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/

/* 

=pod

=head1 NAME

 LipmFullScreen.js - A class useful to go to and from full screen

=cut
*/

/*
=pod

=head1 LipmFullScreen Class      

 Title: LipmFullScreen
 Usage: new LipmFullScreen('tool_id','cl_to_hide','id_to_move',true);
        The object is related to 'tool_id', which must be an img. When you click tool_id:
           -all the elements belonging to 'cl_to_hide' are hidden/shown
           -id_to_move is positioned to (0,0) for full screen, to its previous position for normal screen
           -When the object is created, it goes to full screen if the last constructor parameter is true

 Constructor Args:
           tool_id                                                     = The Id of the tool used to go from/to full screen
                                                                         tool_id should be an img, OR a div contaning an img
           cl_to_hide   (optional, defaults to HideOnFullScreen)       = The class name of the elements we want to hide/show
           id_to_move   (optional, defaults to ID_MOVE_ON_FULL_SCREEN) = the id name of the element we want to move
           init_as_full (optional, default false)                      = If true, switch to full screen when starting
		   
=cut
*/


function basename(path) {
    return path.replace( /.*\//, "" );
};
function dirname(path) {
    var a_tmp = path.match( /.*\// );
	return (a_tmp == null ? '.' : a_tmp[0]);
};
function extension(path) {
    var a_tmp = path.match( /\.[^.]+$/ );
	return (a_tmp == null ? '' : a_tmp[0]);
};

LipmFullScreenClass = Class.create({
	initialize: function(tool_id,cl_to_hide,id_to_move,init_as_full)
	{
		if (cl_to_hide==undefined) {cl_to_hide = "HideOnFullScreen"};
		if (id_to_move==undefined) {id_to_move = "ID_MOVE_ON_FULL_SCREEN"};
		if (init_as_full==undefined) {init_as_full = false};

		this.__tool        = $(tool_id);
		this.__tool.show();
		if (this.__tool.tagName != 'IMG') {this.__tool=this.__tool.down('img');};
		
		/* compute url_full and url_no_full from the current img path:
		   - keep the directory part of the URL and the extension
		   - Use full_screen or no_full_screen as name
		*/
		var ext         = extension(this.__tool.src);
		var dir         = dirname(this.__tool.src);
		this.__url_full = dir + '/full_screen' + ext;
		this.__url_no_full = dir + '/no_full_screen' + ext;
		this.__tool.style.cursor='pointer';
		this.__to_hide  = $$('.'+cl_to_hide);
		this.__to_move  = $(id_to_move);
		this.__hooks    = $A();
		init_as_full = init_as_full || false;

		Event.observe(this.__tool,'click',this.TogglefullScreen.bindAsEventListener(this));
		
		if (init_as_full)
		{
			this.ToFullScreen();
		}
		else
		{
			this.FromFullScreen();
		}
		
	},
	/*
=pod

=head2 RegisterHook

	Title:     RegisterHook
	Usage:     RegisterHook()
	Procedure: Register a function which will be called when we go to or from full screen
	           You may register several functions, they will be called in the registered order
			   The hooks are called with the parameter 'false' when we go From Full Screen, and true when we go To full screen
	Args:      none
	globals:   
	
=cut
	*/

	RegisterHook: function(h)
	{
		this.__hooks.push(h);
	},
	
	/*
=pod

=head2 FromFullScreen

	Title:     FromFullScreen
	Usage:     FromFullScreen()
	Procedure: Display the hidden elements, move the main element back
	Args:      none
	globals:   
	
=cut
	*/
	FromFullScreen: function()
	{
		this.__to_hide.each(function(e) {
			e.show();
		});

		if (this.__to_move != null && this.__left != undefined)
		{
			this.__to_move.setStyle({
				left:  this.__left+'px',
				top: this.__top+'px'
			});
		}
		
		// Call the registered hooks
		this.__hooks.each(function(h) { h.call(false); });
		
		this.__tool.src = this.__url_full;
		this.__full_screen = false;
	},

	/*
=pod

=head2 ToFullScreen

	Title:     ToFullScreen
	Usage:     ToFullScreen()
	Procedure: Hide the marked elements, move the main element 
	Args:      none
	globals:   
	
=cut
	*/
	ToFullScreen: function()
	{
		this.__to_hide.each(function(e) {
			e.hide();
		});
		
		if (this.__to_move != null)
		{
			// Keep in private members the current position of the element to move
			var pos = this.__to_move.viewportOffset();
			this.__left = pos[0];
			this.__top  = pos[1];

			// Put this element at top-left corner
			this.__to_move.setStyle({
				left:'0',
				top: '0'
			});
		};
		
		// Call the registered hooks
		this.__hooks.each(function(h) { h.call(true); });
		
		this.__tool.src = this.__url_no_full;
		this.__full_screen = true;
	},

	/*
=pod

=head2 TogglefullScreen

	Title:     TogglefullScreen
	Usage:     obj.TogglefullScreen()
	Procedure: Go to or from full screen mode
	Args:      none
	
=cut
	*/
	TogglefullScreen: function()
	{
		if (this.__full_screen)
		{
			this.FromFullScreen();
		}
		else
		{
			this.ToFullScreen();
		}
	}
});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

