/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/

/* 

=pod

=head1 NAME
 LipmLed.js - Some indicators ("leds"), 2-state or 3-state

=head1 SYNOPSIS

 LipmLedBaseClass         A base class for indicators
     LipmLedClass             Derives from LipmLedBaseClass, 3-state indicator
     LipmLedPatienceClass     Derives from the LipmLedBaseClass, but only 2-state indicator
     LipmLedCompositeClass    Derives from the LipmLedBaseClass, manages an arbitrary number of leds
	 
=head1 CLASSES AND METHODS

=cut

*/

/*
=pod

=head1 class LipmLedBaseClass

 Title        : LipmLedBaseClass
 Usage        : This base class is used to implement implements a "led" indicator
                The led may have 3 states: off, running, failed

 Constructor arg: initial_status (optional, defaults to off): The initial status of the led
				  
 Member functions:
                SetStatus sets the led status
                GetStatus returns the current status
 Globals      : none

=cut
*/
LipmLedBaseClass = Class.create({
	initialize: function(initial_status)
	{
		if (initial_status==undefined) {initial_status='off';};
		this.__SetStatus(initial_status);
	},
	
	/*
=pod
	
=head2 __SetStatus

    Title: __SetStatus
    Usage: this.__SetStatus('status')
    Procedure: set the status of the object
               The status must be one of the three strings: off, running, failed - If not, the call is ignored
    args:      The status to set
    access:    private
	
=cut
	*/
	
	__SetStatus: function(status)
	{
		if (status!= 'off' && status!='running' && status != 'failed')
		{
			throw("LipmLedBaseclass: Bad status specified " + status);
		}

		this._status = status;
	},
	SetStatus: function(status) {this.__SetStatus(status);},
	GetStatus: function()       {return this._status;}
});

/*
=pod

=head1 class LipmLedClass

 Title        : LipmLedClass
 Usage        : This class implements a "led" indicator
                The led may have 3 status:
                    -off     (a black or dark blue led)                 
                    -running (generally a yellow led)
                    -failed  (generally a red led)

 Constructor arg: led_id: the Id of an html image to change when the status is changed
                          There should be three files in the same directory, the three files should have the same name except from
                          _off, _running, _failed (followed by the extension: .png or .gif). Thus, following names are correct:
                          /path/to/some/led_off.png, /path/to/some/led_running.png, /path/to/some/led_failed.png
                  initial_status (optional, defaults to off): The initial status of the led
				  
 Member functions:
                SetStatus sets the led status
                GetStatus returns the current status
 Globals      : none

=cut
*/
LipmLedClass = Class.create(LipmLedBaseClass,{
	initialize: function($super,led_id,initial_status)
	{
		$super(initial_status);
		this._led      = $(led_id);
		if (this._led == null) {throw('ERROR - LipmLedClass could not find ' + led_id);};
		
		// could be a <div> with an img inside
		if (this._led.tagName != 'IMG') { this._led = this._led.down('img'); };
		if (this._led == null) {throw('ERROR - LipmLedClass could not find any img under ' + led_id);};

		this._template = this._led.src;
		if (this._template.endsWith('.png')) {this._template = this._template.gsub(/_[a-z]+\.png$/,'_xxxxx.png');};
		if (this._template.endsWith('.gif')) {this._template = this._template.gsub(/_[a-z]+\.gif$/,'_xxxxx.gif');};
	},
	
	/*
=pod
	
=head2 SetStatus

    Title: SetStatus
    Usage: obj.SetStatus('status')
    Procedure: set the status of the object
               The status must be one of the three strings: off, running, failed - If not, the call is ignored
    args:      The status to set
	
=cut
	*/
	
	SetStatus: function($super,status)
	{
		$super(status);

		var img_src = this._template.gsub('xxxxx',status);
		this._led.src = img_src;
	}
});

/*
=pod

=head1 class LipmLedPatienceClass

 Title        : LipmLedPatienceClass
 Usage        : This class derives from LipmLedClass, and implements a two-state indicator, may be using the wb_patience from Webbuilder
                The led may have 2 status:
                    -off/failed     (the indicator is hidden)
                    -running        (the indicator is shown) 

 Constructor arg: led_id        : the Id of an html image to hide/show (wb_patience for instance)
                  initial_status: The initial status of the led

 Member functions:
                SetStatus sets the led status
                GetStatus returns the current status
 Globals      : none

=cut
*/

LipmLedPatienceClass = Class.create(LipmLedBaseClass, {
	initialize: function($super,led_id,initial_status)
	{
		$super(initial_status);
		this._led = $(led_id);
	},
	
	/*
=pod

=head2 SetStatus

     Title: SetStatus
     Usage: obj.SetStatus('status')
     Procedure: set the status of the object
                The status must be one of the three strings: off, running, failed - If not, the call is ignored
                obj.SetStatus('off') and obj.SetStauts('failed') produce the same effect
     args:      the status to set
	
=cut
	*/
	
	SetStatus: function($super,status)
	{
		$super(status);
		if (status == 'running')
		{
			this._led.show();
		}
		else
		{
			this._led.hide();
		}
	}
});
	
/*
=pod

=head1 class LipmLedCompositeClass

 Title        : LipmLedCompositeClass
 Usage        : This class manages an arbitrary number of objects, all deriving from LipmLedBaseClass
                All leds are synchronized, ie they have the same status

 Constructor arg: initial_status: The initial status of the leds
				  
 Member functions:
                RegisterLed register a new led to be managed by LipmLedCompositeClass
                SetStatus sets the led status
                GetStatus returns the current status
 Globals      : none

=cut
*/

LipmLedCompositeClass = Class.create(LipmLedBaseClass, {
	initialize: function($super,initial_status)
	{
		$super(initial_status);
		this.__a_managed_leds=$A();
	},

	/*
=pod

=head2 RegisterLed

     Title: RegisterLed
     Usage: obj.RegisterLed(o_led)
     Procedure: register a new led to be managed
     args:    o_led must be an already initialized object, deriveing from LipmLedBaseClass
	
=cut
	*/
	RegisterLed: function (o_led)
	{
		o_led.SetStatus(this._status);
		this.__a_managed_leds.push(o_led);
	},

	/*
=pod

=head2 SetStatus

     Title: SetStatus
     Usage: obj.SetStatus('status')
     Procedure: set the status of each managed led
     args:    the status to set
	
=cut
	*/
	SetStatus: function($super,status)
	{
		$super(status);
		for (var i=0; i<this.__a_managed_leds.length; ++i)
		{
			this.__a_managed_leds[i].SetStatus(status);
		}
	}
});
	
/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

